"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const heroku_cli_util_1 = require("heroku-cli-util");
const graphql_1 = require("graphql");
const chalk_1 = require("chalk");
const git_1 = require("../../git");
const Command_1 = require("../../Command");
const utils_1 = require("../../utils");
const graphqlTypes_1 = require("apollo-language-server/lib/graphqlTypes");
const formatChange = (change) => {
    let color = (x) => x;
    if (change.type === graphqlTypes_1.ChangeType.FAILURE) {
        color = chalk_1.default.red;
    }
    if (change.type === graphqlTypes_1.ChangeType.WARNING) {
        color = chalk_1.default.yellow;
    }
    return {
        type: color(change.type),
        code: color(change.code),
        description: color(change.description)
    };
};
class ServiceCheck extends Command_1.ProjectCommand {
    async run() {
        const { gitContext, checkSchemaResult, shouldOutputJson } = await this.runTasks(({ config, flags, project }) => [
            {
                title: "Checking service for changes",
                task: async (ctx) => {
                    if (!config.name) {
                        throw new Error("No service found to link to Engine");
                    }
                    const tag = flags.tag || config.tag || "current";
                    const schema = await project.resolveSchema({ tag });
                    ctx.gitContext = await git_1.gitInfo(this.log);
                    const historicParameters = utils_1.validateHistoricParams({
                        validationPeriod: flags.validationPeriod,
                        queryCountThreshold: flags.queryCountThreshold,
                        queryCountThresholdPercentage: flags.queryCountThresholdPercentage
                    });
                    ctx.checkSchemaResult = await project.engine.checkSchema(Object.assign({ id: config.name, schema: graphql_1.introspectionFromSchema(schema).__schema, tag: flags.tag, gitContext: ctx.gitContext, frontend: flags.frontend || config.engine.frontend }, (historicParameters && { historicParameters })));
                    ctx.shouldOutputJson = flags.json;
                }
            }
        ]);
        const { targetUrl, diffToPrevious: { changes, validationConfig } } = checkSchemaResult;
        const failures = changes.filter(({ type }) => type === graphqlTypes_1.ChangeType.FAILURE);
        if (shouldOutputJson) {
            return this.log(JSON.stringify({ targetUrl, changes, validationConfig }, null, 2));
        }
        if (changes.length === 0) {
            return this.log("\nNo changes present between schemas\n");
        }
        this.log("\n");
        heroku_cli_util_1.table(changes.map(formatChange), {
            columns: [
                { key: "type", label: "Change" },
                { key: "code", label: "Code" },
                { key: "description", label: "Description" }
            ]
        });
        this.log("\n");
        if (targetUrl) {
            this.log(`View full details at: ${targetUrl}`);
        }
        if (failures.length > 0) {
            this.exit();
        }
        return;
    }
}
ServiceCheck.aliases = ["schema:check"];
ServiceCheck.description = "Check a service against known operation workloads to find breaking changes";
ServiceCheck.flags = Object.assign({}, Command_1.ProjectCommand.flags, { tag: command_1.flags.string({
        char: "t",
        description: "The published tag to check this service against"
    }), validationPeriod: command_1.flags.string({
        description: "The size of the time window with which to validate the schema against. You may provide a number (in seconds), or an ISO8601 format duration for more granularity (see: https://en.wikipedia.org/wiki/ISO_8601#Durations)"
    }), queryCountThreshold: command_1.flags.integer({
        description: "Minimum number of requests within the requested time window for a query to be considered."
    }), queryCountThresholdPercentage: command_1.flags.integer({
        description: "Number of requests within the requested time window for a query to be considered, relative to total request count. Expected values are between 0 and 0.05 (minimum 5% of total request volume)"
    }), json: command_1.flags.boolean({
        description: "Output result in json, which can then be parsed by CLI tools such as jq."
    }) });
exports.default = ServiceCheck;
//# sourceMappingURL=check.js.map